@@ -18,6 +18,7 @@
 #include "csvimportdlg.h"
 #include "printersettings.h"
 #include "sqltables.h"
+#include "encodingcombo.h"
 
 // Qt includes
 #include <qfile.h>
@@ -37,6 +38,7 @@
 #include <knuminput.h>
 #include <kpushbutton.h>
 #include <kurlrequester.h>
+#include "csvfile.h"
 
 // import from labelprinter.cpp
 extern QString removeQuote( QString text, QString quote );
@@ -51,8 +53,9 @@ CSVImportDlg::CSVImportDlg(QWidget *parent, const char *name )
     
     QFrame* box = plainPage();
     QVBoxLayout* layout = new QVBoxLayout( box, 6, 6 );
-    
+
     requester = new KURLRequester( box );
+    comboEncoding = new EncodingCombo( box );
     comboSQL = new KComboBox( false, box );
     comboSQL->insertItem( TABLE_BASIC );
     comboSQL->insertItem( TABLE_CUSTOMER );
@@ -79,6 +82,8 @@ CSVImportDlg::CSVImportDlg(QWidget *parent, const char *name )
     
     layout->addWidget( new QLabel( i18n("File to import:"), box ) );
     layout->addWidget( requester );
+    layout->addWidget( new QLabel( i18n("Encoding:"), box ) );
+    layout->addWidget( comboEncoding );
     layout->addWidget( new QLabel( i18n("Import into table:"), box ) );
     layout->addWidget( comboSQL );
     layout->addWidget( table );
@@ -88,6 +93,7 @@ CSVImportDlg::CSVImportDlg(QWidget *parent, const char *name )
     connect( requester, SIGNAL( textChanged( const QString & ) ), this, SLOT( settingsChanged() ) );
     connect( buttonSet, SIGNAL( clicked() ), this, SLOT( setCol() ) );
     connect( comboSQL, SIGNAL( activated( int ) ), this, SLOT( updateFields() ) );
+    connect( comboEncoding, SIGNAL( activated( int ) ), this, SLOT( settingsChanged() ) );
     connect( table->horizontalHeader(), SIGNAL( clicked( int ) ), this, SLOT( updateCol( int ) ) );
 
     updateFields();
@@ -102,52 +108,37 @@ CSVImportDlg::~CSVImportDlg()
 
 void CSVImportDlg::settingsChanged()
 {
-    QFile file( requester->url() );
-    if( !file.open( IO_ReadOnly ) ) {
-        table->setNumCols( 0 );
-        table->setNumRows( 0 );
-        return;
-    }
-    
-    labelprinterdata* lpdata = PrinterSettings::getInstance()->getData();
-        
-    QTextStream t( &file );
-    table->setNumCols( 1 );
+    CSVFile file( requester->url() );
+    QStringList list;
     int i = 0;
-    while( !t.atEnd() ) {
-        QString s = t.readLine();
+    unsigned int z;
 
-        // same as in labelprinter.cpp
-        s = s.stripWhiteSpace();
-        if( s.left( lpdata->comment.length() ) == lpdata->comment && !lpdata->comment.isEmpty() )
-            /* Line is a comment */
-            continue;
+    file.setEncoding( comboEncoding->currentText() );
 
-        if( s.isEmpty() )
-            continue;
+    table->setNumCols( 0 );
+    table->setNumRows( 0 );
 
-        // if line does not end with separator, add one
-        if( s.right( 1 ) != lpdata->separator ) 
-            s.append( lpdata->separator );
-            
-        // calculate the number of rows
-        int c = s.contains( lpdata->separator );
-        if( table->numCols() < c )
-            table->setNumCols( c );
+    if( !file.isValid() )
+        return;
+     
+    while( !file.isEof() )
+    {
+        list = file.readNextLine();
+
+        if( table->numCols() < (int)list.count() )
+            table->setNumCols( list.count() );
             
         if( table->numRows() <= i )
             // add 100 rows to get a reasonable speed
             table->setNumRows( i + 100 );
 
-        for( int z = 0; z < c; z++ )
-            table->setText( i, z, removeQuote( s.section( lpdata->separator, z, z ), lpdata->quote ) );
+        for( z = 0; z < list.count(); z++ )
+            table->setText( i, z, list[z] );
             
         i++;
     }
-
+    
     table->setNumRows( i );
-    file.close();
-
     spinCol->setRange( 1, table->numCols(), 1, false );
         
     enableControls();
@@ -235,3 +226,5 @@ void CSVImportDlg::accept()
 }
 
 
+
+#include "csvimportdlg.moc"
