@@ -1,7 +1,7 @@
 /***************************************************************************
                           dsmainwindow.cpp  -  description
                              -------------------
-    begin                : Fre Jan 17 2003
+    begin                : Fri Jan 17 2003
     copyright            : (C) 2003 by Dominik Seichter
     email                : domseichter@web.de
  ***************************************************************************/
@@ -20,7 +20,8 @@
 #include "confwizard.h"
 #include "printersettings.h"
 #include "kbarcodesettings.h"
-#include "mybarcode.h"
+#include "kactionmap.h"
+#include "barkode.h"
 
 // Qt includes
 #include <qcheckbox.h>
@@ -90,6 +91,8 @@ void DSMainWindow::setupActions()
     hlpMenu->removeItem( helpid );
     hlpMenu->insertItem( SmallIconSet("contents"), i18n("&Help"),
         this, SLOT(appHelpActivated()), Key_F1, -1, 0 );
+    hlpMenu->insertSeparator(-1);
+    hlpMenu->insertItem( i18n("&Action Map..."), this, SLOT( slotFunctionMap() ), 0, -1, 0 );
     hlpMenu->insertSeparator(-2);
     hlpMenu->insertItem( SmallIconSet("system"), i18n("&System Check..."), this, SLOT(showCheck() ), 0, -1, 0 );
     hlpMenu->insertItem( SmallIconSet("barcode"), i18n("&Barcode Help..."), this, SLOT( startInfo() ), 0, -1, 0 );
@@ -111,7 +114,7 @@ void DSMainWindow::setupActions()
     importExampleAct->plug( settings );
 
     SqlTables* tables = SqlTables::getInstance();
-    if( tables->getData()->autoconnect && autoconnect && !first ) {
+    if( tables->getData().autoconnect && autoconnect && !first ) {
         tables->connectMySQL();
         autoconnect = false;
     }
@@ -129,7 +132,7 @@ void DSMainWindow::loadConfig()
     first = config->readBoolEntry("firststart2", true );
 
     SqlTables* tables = SqlTables::getInstance();
-    if( tables->getData()->autoconnect && !first && autoconnect && connectAct ) {
+    if( tables->getData().autoconnect && !first && autoconnect && connectAct ) {
         connectAct->setEnabled( !tables->connectMySQL() );
         importLabelDefAct->setEnabled( !connectAct->isEnabled() );
         importExampleAct->setEnabled( !connectAct->isEnabled() );
@@ -216,9 +219,10 @@ void DSMainWindow::donations()
 
 QString DSMainWindow::systemCheck()
 {
-    bool gnubarcode = !BarCode::haveGNUBarcode();
-    bool pdf = !BarCode::havePDFBarcode();
-    bool tbarcode = !BarCode::haveTBarcode();
+    bool gnubarcode = !Barkode::haveGNUBarcode();
+    bool pdf = !Barkode::havePDFBarcode();
+    bool tbarcode = !Barkode::haveTBarcode();
+    bool pure = !Barkode::havePurePostscriptBarcode();
 
     QString text;
 
@@ -229,6 +233,8 @@ QString DSMainWindow::systemCheck()
     text.append( pdf ? i18n("<b>No</b><br />") : i18n("<b>Found</b><br />") );
     text.append( "TBarcode: ");
     text.append( tbarcode ? i18n("<b>No</b><br />") : i18n("<b>Found</b><br />") );
+    text.append( "Pure Postscript Barcode Writer: ");
+    text.append( pure ? i18n("<b>No</b><br />") : i18n("<b>Found</b><br />") );
 
     if( gnubarcode && tbarcode && pdf )
         text.append( i18n("<p>To get <b>barcode support</b> you have to either install <i>GNU Barcode</i>, <i>TBarcode</i> or <i>PDF417 Enc</i>.</p>") );
@@ -250,3 +256,10 @@ QString DSMainWindow::systemCheck()
 
     return text;
 }
+
+void DSMainWindow::slotFunctionMap()
+{
+    new KActionMapDlg( actionCollection(), this );
+}
+
+#include "dsmainwindow.moc"
