@@ -1,4 +1,4 @@
-/***************************************************************************
+f/***************************************************************************
                           barcode.cpp  -  description
                              -------------------
     begin                : Die Apr 23 2002
@@ -15,10 +15,10 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "mybarcode.h"
 #include "sqltables.h"
 #include "barcodecache.h"
 
+#include "mybarcode.h"
 #include <stdio.h>
 
 // Qt includes
@@ -152,7 +152,7 @@ bool BarCode::createPixmap( QPixmap* target, int resx, int resy )
     }
 
     target->load( input->name(), "PBM" );
-    BarcodeCache::instance()->write( barcode, resx, resy, target, m_value );
+//    BarcodeCache::instance()->write( barcode, resx, resy, target, m_value );
 
     input->unlink();
     output->unlink();
@@ -170,7 +170,7 @@ void BarCode::createBarcode( QPixmap* target, QPaintDevice* device )
     int resy = pdm.logicalDpiY();
 
     QString value = createSequence( barcode.value );
-    QPixmap* cached = BarcodeCache::instance()->read( barcode, resx, resy, value );
+    QPixmap* cached = 0;//BarcodeCache::instance()->read( barcode, resx, resy, value );
 
     // no matching barcode found in cache
     if( !cached ) {
@@ -546,6 +546,7 @@ void BarCode::setHaveBarcode()
             codes.append( fillStruct( "i25 -c", i18n("interleaved 2 of 5 (only digits, no checksum)"), GNU_BARCODE ) );
             codes.append( fillStruct( "128raw", i18n("Raw code 128"), GNU_BARCODE ) );
             codes.append( fillStruct( "cbr", i18n("Codabar"), GNU_BARCODE ) );
+            codes.append( fillStruct( "cbr -c", i18n("Codabar (no checksum)"), GNU_BARCODE ) );            
             codes.append( fillStruct( "msi", i18n("MSI"), GNU_BARCODE ) );
             codes.append( fillStruct( "pls", i18n("Plessey"), GNU_BARCODE ) );
             codes.append( fillStruct( "code93", i18n("Code 93"), GNU_BARCODE ) );
