@@ -65,14 +65,10 @@ void MyCanvas::drawBackground( QPainter & painter, const QRect & clip )
             painter.translate( m_rect.x(), m_rect.y() );
 
             painter.setPen( QPen( KBarcodeSettings::getInstance()->gridColor(), 0 ) );
-            for( int x = 0; x <= m_rect.width(); x += KBarcodeSettings::getInstance()->gridSize() )
-                for( int y = 0; y <= m_rect.height(); y += KBarcodeSettings::getInstance()->gridSize() ) {
-//                    if( clip.x() < x && clip.x() + clip.height() > x )
-                        painter.drawLine( x, 0, x, m_rect.height() );
+	    for( int y = 0; y <= m_rect.height(); y += KBarcodeSettings::getInstance()->gridSize() ) 
+		for( int x = 0; x <= m_rect.width(); x += KBarcodeSettings::getInstance()->gridSize() )
+		    painter.drawPoint( x, y );
 
-//                    if( clip.y() < y && clip.y() + clip.width() > y )
-                        painter.drawLine( 0, y, m_rect.width(), y );
-                }
             painter.translate( -m_rect.x(), -m_rect.y() );
         }
     }
@@ -105,14 +101,43 @@ MyCanvasView::~MyCanvasView()
 {
 }
 
-void MyCanvasView::snapPoint(QPoint * point)
+void MyCanvasView::snapPoint(QPoint * point, TCanvasItem* item )
 {
     // move item to next grid position
     // TODO: align the top left to the grid not the current mouse position!
-    point->setX(point->x() + KBarcodeSettings::getInstance()->gridSize()/2);
-    point->setY(point->y() + KBarcodeSettings::getInstance()->gridSize()/2);
-    point->setX(point->x() - point->x() % KBarcodeSettings::getInstance()->gridSize());
-    point->setY(point->y() - point->y() % KBarcodeSettings::getInstance()->gridSize());
+
+    /*
+    */
+
+    if( item )
+    {
+	int difx = point->x() - (int)item->x();
+	int dify = point->y() - (int)item->y();
+
+	int x = (int)item->x();
+	int y = (int)item->y();
+
+	int grid = KBarcodeSettings::getInstance()->gridSize();
+	int modx = x % grid;
+	int mody = y % grid;
+
+        x -= modx;
+	y -= mody;
+
+	if(modx >= grid / 2)
+	    x += grid;
+
+	if(mody >= grid / 2)
+	    y += grid;
+
+	point->setX( x + difx );
+	point->setY( y + dify );
+    }
+    else
+    {
+	point->setX(point->x() - point->x() % KBarcodeSettings::getInstance()->gridSize());
+	point->setY(point->y() - point->y() % KBarcodeSettings::getInstance()->gridSize());
+    }
 }
 
 void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
@@ -139,72 +164,69 @@ void MyCanvasView::contentsMouseMoveEvent(QMouseEvent* e)
 
     bool shift_pressed = e->state() & Qt::ShiftButton;
 
-    MyCanvasRectangle* moving = getActive();
-    if( moving ) {
+    TCanvasItem* moving = getActive();
+    if( moving && !moving->item()->locked() ) {
         QPoint p = inverseWorldMatrix().map(e->pos());
 
         if( m_mode == Barcode || m_mode == Inside ) {
-            QCanvasItemList list = getSelected();
+            TCanvasItemList list = getSelected();
             for( unsigned int i = 0; i < list.count(); i++ ) {
-                MyCanvasRectangle* moving = (MyCanvasRectangle*)list[i];
-                QPoint new_pt =QPoint(p.x() - delta_pt.x(),p.y() - delta_pt.y());
+                TCanvasItem* moving = list[i];                    
+                QPoint new_pt = QPoint(p.x() - delta_pt.x(),p.y() - delta_pt.y());
                 if( canv->grid() ) {
-                    snapPoint(&new_pt) ;
+                    snapPoint(&new_pt, moving) ;
                 }
+
+		LabelUtils l;
+		QPoint pmm;
+		pmm.setX( (int)l.pixelToMm( new_pt.x() - getTranslation().x(), this, LabelUtils::DpiX ) * 1000 );
+		pmm.setY( (int)l.pixelToMm( new_pt.y() - getTranslation().y(), this, LabelUtils::DpiY ) * 1000 );
+
                 // Move the item
-                MoveCommand* mv = new MoveCommand( new_pt.x() - (int) moving->x(),
-                                                   new_pt.y() - (int) moving->y(), moving );
+                MoveCommand* mv = new MoveCommand( pmm.x() - moving->item()->rectMM().x(),
+                                                   pmm.y() - moving->item()->rectMM().y(), moving );
                 mv->execute();
                 getMoveCommand()->addCommand( mv );
             }
-        } else if( m_mode == LineLeft || m_mode == LineRight ) {
-            QPoint a, b;
-            MyCanvasLine* line = (MyCanvasLine*)moving;
-
-            if( canv->grid() ) {
-                snapPoint(&p) ;
-            }
-            
-            a = m_mode == LineLeft ? p : line->startPoint();
-            b = m_mode == LineRight ? p : line->endPoint();
-
-            LineMoveCommand* lmc = new LineMoveCommand( a, b, line );
-            lmc->execute();
-
-            getMoveCommand()->addCommand( lmc );
-            repaintContents( line->boundingRect() );
         } else {
             
             if( canv->grid() ) {
-                snapPoint(&p) ;
+                snapPoint(&p, NULL ) ;
             }
             
+	    LabelUtils l;
+
+	    QPoint pmm;
+	    pmm.setX( (int)l.pixelToMm( p.x() - getTranslation().x(), this, LabelUtils::DpiX ) * 1000 );
+	    pmm.setY( (int)l.pixelToMm( p.y() - getTranslation().y(), this, LabelUtils::DpiY ) * 1000 );
+
+	    QRect rmm = moving->item()->rectMM();
             ResizeCommand* mv = new ResizeCommand( moving, shift_pressed );
-            
+
             switch( m_mode ) {
                 case RightMiddle:
-                    mv->setRect( (int) moving->x(), (int) moving->y(), p.x() - (int) moving->x(), moving->height() );
+                    mv->setRect( rmm.x(), rmm.y(), pmm.x() - rmm.x(), rmm.height() );
                     break;
                 case LeftMiddle:
-                    mv->setRect( p.x(), (int) moving->y(), (int) moving->width()+ ((int) moving->x() - p.x()), (int) moving->height() );
+                    mv->setRect( pmm.x(), rmm.y(),  rmm.width()+ (rmm.x() - pmm.x()), rmm.height() );
                     break;
                 case BottomMiddle:
-                    mv->setRect( (int) moving->x(), (int) moving->y(), (int) moving->width(), p.y() - (int) moving->y());
+                    mv->setRect( rmm.x(), rmm.y(), rmm.width(), pmm.y() - rmm.y());
                     break;
                 case TopMiddle:
-                    mv->setRect( (int) moving->x(), p.y(), (int) moving->width(), (int) moving->height()+ ((int) moving->y() - p.y()));
+                    mv->setRect( rmm.x(), pmm.y(), rmm.width(), rmm.height()+ (rmm.y() - pmm.y()));
                     break;
                 case BottomLeft:
-                    mv->setRect( p.x(), (int) moving->y(), (int) moving->width()+ ((int) moving->x() - p.x()), p.y() - (int) moving->y() );
+                    mv->setRect( pmm.x(), rmm.y(), rmm.width()+ (rmm.x() - pmm.x()), pmm.y() - rmm.y() );
                     break;
                 case BottomRight:
-                    mv->setRect( (int) moving->x(), (int) moving->y(), p.x() - (int) moving->x(), p.y() - (int) moving->y() );
+                    mv->setRect( rmm.x(), rmm.y(), pmm.x() - rmm.x(), pmm.y() - rmm.y() );
                     break;
                 case TopLeft:
-                    mv->setRect( p.x(), p.y(), (int) moving->width()+ ((int) moving->x() - p.x()), (int) moving->height()+ ((int) moving->y() - p.y()));
+                    mv->setRect( pmm.x(), pmm.y(), rmm.width()+ (rmm.x() - pmm.x()), rmm.height()+ (rmm.y() - pmm.y()));
                     break;
                 case TopRight:
-                    mv->setRect( (int) moving->x(), p.y(), p.x() - (int) moving->x(),(int) moving->height()+ ((int) moving->y() - p.y()) );
+                    mv->setRect( rmm.x(), pmm.y(), pmm.x() - rmm.x(),rmm.height()+ (rmm.y() - pmm.y()) );
                     break;
                 default:
                     break;
@@ -227,7 +249,7 @@ void MyCanvasView::contentsMousePressEvent(QMouseEvent* e)
     for( int z = MyCanvasView::getLowestZ( list ); z <= MyCanvasView::getHighestZ( list ); z++ )
         for( unsigned int i = 0; i < list.count(); i++ )
             if( list[i]->z() == z && isInside( e->pos(), list[i] ) )
-                setActive( (MyCanvasRectangle*)list[i], (e->state() & Qt::ControlButton) );
+                setActive( list[i], (e->state() & Qt::ControlButton) );
 
     if( getActive() ) {
         moving_start = inverseWorldMatrix().map(e->pos());
@@ -237,7 +259,7 @@ void MyCanvasView::contentsMousePressEvent(QMouseEvent* e)
     }
 
     if( e->button() == Qt::RightButton && getActive() )
-        emit showContextMenu( getActive(), e->globalPos() );
+        emit showContextMenu( e->globalPos() );
 }
 
 void MyCanvasView::contentsMouseReleaseEvent(QMouseEvent* e)
@@ -250,10 +272,6 @@ void MyCanvasView::contentsMouseReleaseEvent(QMouseEvent* e)
         m_commov = 0;
     }
 
-    // TODO: check: is this needed
-    if( m_mode == LineLeft || m_mode == LineRight )
-        repaintContents();
-
     updateCursor( e->pos() );
 }
 
@@ -272,7 +290,7 @@ void MyCanvasView::contentsMouseDoubleClickEvent(QMouseEvent* e)
     for( int z = MyCanvasView::getHighestZ( list ); z >= MyCanvasView::getLowestZ( list ); z-- )    
         for( unsigned int i = 0; i < list.count(); i++ )
             if( list[i]->z() == z && isInside( e->pos(), list[i] ) ) {
-                setActive( (MyCanvasRectangle*)list[i] );
+                setActive( list[i] );
                 emit doubleClickedItem( getActive() );
                 return;
             }
@@ -291,21 +309,6 @@ int MyCanvasView::isEdge( QPoint p, QCanvasItem* item )
     if( !isInside( p, item ) )
         return Outside;
 
-    if( item->rtti() == LINE_RTTI ) {
-        MyCanvasLine* line = (MyCanvasLine*)item;
-
-        QRect start( line->startPoint(), QSize( SPOTSIZE, SPOTSIZE ) );
-        QRect end( line->endPoint(), QSize( SPOTSIZE, SPOTSIZE ) );
-
-        if( start.contains( p ) )
-            return LineLeft;
-
-        if( end.contains( p ) )
-            return LineRight;
-
-        return Inside;
-    }
-
     QRect r = item->boundingRect();
     
     int rh = r.y() + r.height();
@@ -368,10 +371,10 @@ int MyCanvasView::isEdge( QPoint p, QCanvasItem* item )
 
 void MyCanvasView::deleteCurrent()
 {
-    if( getActive() ) {
+    TCanvasItemList list = getSelected();
+    if( !list.isEmpty() ) {
         KMacroCommand* mc = new KMacroCommand( i18n("Delete") );
 
-        QCanvasItemList list = getSelected();
         for( unsigned int i = 0; i < list.count(); i++ ) {
             DeleteCommand* dc = new DeleteCommand( list[i] );
             dc->execute();
@@ -396,8 +399,7 @@ void MyCanvasView::updateRuler()
     rulerv->setGeometry( 0, 20, 20, height() - 20 );
 
     if( def ) {
-        Measurements* m = def->getMeasurements();
-        canv->setRect( QRect( translation.x(), translation.y(), (int)m->width( this ), (int)m->height( this )) );
+        canv->setRect( QRect( translation.x(), translation.y(), (int)def->getMeasurements().width( this ), (int)def->getMeasurements().height( this )) );
 
         rulerv->setMaxValue( height() );
         rulerh->setMaxValue( width() );
@@ -417,12 +419,6 @@ void MyCanvasView::resizeEvent( QResizeEvent * r )
     setUpdatesEnabled( false );
     QPoint old = translation;
 
-    Measurements* measure = def->getMeasurements();
-    // make sure that there is some space around the label, therefore at 60 pixel
-    int w = ( this->width() - 2 > measure->width( this ) ? this->width() - 2 : int(measure->width( this ) + 60) );
-    int h = ( this->height() - 2 > measure->height( this ) ? this->height() - 2 : int(measure->height( this ) + 60) );
-
-    canvas()->resize( w, h );
     QCanvasView::resizeEvent( r );
 
     reposition();
@@ -447,15 +443,20 @@ void MyCanvasView::reposition()
      * and multiply with the new one.
      * As a result we have the correct width.
      */
-
-    Measurements* measure = def->getMeasurements();
-    int x = int((width() - (measure->width( this ))  ) / 2);
-    int y = int((height() - (measure->height( this )) ) / 2);
+    int x = int((width() - (def->getMeasurements().width( this ))  ) / 2);
+    int y = int((height() - (def->getMeasurements().height( this )) ) / 2);
 
     // move label 30 pixels away from top/left if necessary
     x = x > 0 ? x : 30;
     y = y > 0 ? y : 30;
 
+    // make sure that there is some space around the label, therefore at 60 pixel
+    int w = ( this->width() - 2 > def->getMeasurements().width( this ) ? this->width() - 2 : int(def->getMeasurements().width( this ) + 60) );
+    int h = ( this->height() - 2 > def->getMeasurements().height( this ) ? this->height() - 2 : int(def->getMeasurements().height( this ) + 60) );
+
+    canvas()->resize( w, h );
+    
+    
     translation = QPoint( x, y );
 }
 
@@ -467,6 +468,17 @@ void MyCanvasView::setDefinition( Definition* d )
     repaintContents( true );
 }
 
+void MyCanvasView::selectAll()
+{
+    QCanvasItemList list = canvas()->allItems();
+    for( unsigned int i = 0; i < list.count(); i++ )
+        setSelected( list[i], true );
+}
+
+void MyCanvasView::deSelectAll()
+{
+    setSelected( 0 );
+}
 
 int MyCanvasView::getLowestZ( QCanvasItemList list )
 {
@@ -488,12 +500,12 @@ int MyCanvasView::getHighestZ( QCanvasItemList list )
     return v;
 }
 
-MyCanvasRectangle* MyCanvasView::getActive()
+TCanvasItem* MyCanvasView::getActive()
 {
     QCanvasItemList list = canvas()->allItems();
     for( unsigned int i = 0; i < list.count(); i++ )
         if( list[i]->isActive() )
-            return (MyCanvasRectangle*)list[i];
+            return (TCanvasItem*)list[i];
 
     return 0;
 }
@@ -511,13 +523,24 @@ void MyCanvasView::setActive( QCanvasItem* item, bool control )
     setSelected( item, control );
 }
 
-QCanvasItemList MyCanvasView::getSelected()
+DocumentItemList MyCanvasView::getAllItems()
 {
-    QCanvasItemList l;
+    DocumentItemList l;
+
+    QCanvasItemList list = canvas()->allItems();
+    for( unsigned int i = 0; i < list.count(); i++ )
+	l.append( ((TCanvasItem*)list[i])->item() );
+
+    return l;
+}
+
+TCanvasItemList MyCanvasView::getSelected()
+{
+    TCanvasItemList l;
     QCanvasItemList list = canvas()->allItems();
     for( unsigned int i = 0; i < list.count(); i++ )
         if( list[i]->isSelected() )
-            l.append( list[i] );
+            l.append( (TCanvasItem*)list[i] );
 
     return l;
 }
@@ -540,9 +563,16 @@ int MyCanvasView::updateCursor( QPoint pos, bool pressed )
         this->setCursor( QCursor::ArrowCursor );
         return -1;
     }
-
+    
     int mode = isEdge( pos, getActive() );
-    if( getActive()->rtti() == BARCODE_RTTI ) {
+
+    if( getActive()->item()->locked() )
+    {
+        pressed && mode != Outside ? setCursor( Qt::ForbiddenCursor ) : setCursor( QCursor::ArrowCursor );
+        return mode;        
+    }
+
+    if( getActive()->rtti() == eRtti_Barcode ) {
         pressed && mode != Outside ? setCursor( QCursor::SizeAllCursor ) : setCursor( QCursor::ArrowCursor );
         mode = Barcode;
         return mode;
@@ -561,8 +591,6 @@ int MyCanvasView::updateCursor( QPoint pos, bool pressed )
         case BottomLeft:
             this->setCursor( QCursor::SizeBDiagCursor );
             break;
-        case LineLeft:
-        case LineRight:
         case RightMiddle:
         case LeftMiddle:
             this->setCursor( QCursor::SizeHorCursor );
@@ -579,3 +607,5 @@ int MyCanvasView::updateCursor( QPoint pos, bool pressed )
     return mode;
 }
 
+
+#include "mycanvasview.moc"
