@@ -79,8 +79,7 @@ SqlTables::SqlTables( QObject* parent )
     drivers.insert( "sqlite", new SQLiteDescription() );
     drivers.insert( "QODBC3", new SQLiteDescription() );
 
-    db = 0;
-    sqldata = new mysqldata;
+    db = NULL;
     connected = false;
     loadConfig();
 
@@ -91,7 +90,7 @@ SqlTables::~SqlTables()
 {
 }
 
-SqlTables* SqlTables::instance = 0;
+SqlTables* SqlTables::instance = NULL;
 
 SqlTables* SqlTables::getInstance()
 {
@@ -103,19 +102,19 @@ SqlTables* SqlTables::getInstance()
 
 const SqlDescription* SqlTables::driver() const
 {
-    return drivers[sqldata->driver];
+    return drivers[sqldata.driver];
 }
 bool SqlTables::connectMySQL()
 {
-    db = QSqlDatabase::addDatabase( sqldata->driver );
+    db = QSqlDatabase::addDatabase( sqldata.driver );
 
-    db->setDatabaseName( sqldata->database );
-    db->setUserName( sqldata->username );
-    db->setPassword( sqldata->password );
-    db->setHostName( sqldata->hostname );
+    db->setDatabaseName( sqldata.database );
+    db->setUserName( sqldata.username );
+    db->setPassword( sqldata.password );
+    db->setHostName( sqldata.hostname );
 
     if( !db->open() )
-        KMessageBox::error( 0, i18n("<qt>Unable to open database: ") + sqldata->database + "<br>" +
+        KMessageBox::error( 0, i18n("<qt>Unable to open database: ") + sqldata.database + "<br>" +
               db->lastError().databaseText() + "</qt>");
 
     connected = db->open();
@@ -131,7 +130,7 @@ bool SqlTables::connectMySQL()
 
 bool SqlTables::newTables()
 {
-    return newTables( sqldata->username, sqldata->password, sqldata->hostname, sqldata->database, sqldata->driver );
+    return newTables( sqldata.username, sqldata.password, sqldata.hostname, sqldata.database, sqldata.driver );
 }
 
 bool SqlTables::newTables( const QString & username, const QString & password, const QString & hostname, const QString & database, const QString & driver )
@@ -141,7 +140,9 @@ bool SqlTables::newTables( const QString & username, const QString & password, c
                           QString( TABLE_BASIC "', '" TABLE_CUSTOMER "', '"
                           TABLE_CUSTOMER_TEXT) +i18n("' and '") + QString(TABLE_LABEL_DEF "'")) ==  KMessageBox::Cancel )
         return false;
-
+	if( !drivers[driver] )
+			return false;
+	
     QSqlDatabase*dbase = QSqlDatabase::addDatabase(driver, drivers[driver]->initdb( database ) );
     dbase->setDatabaseName( drivers[driver]->initdb( database ) );
     dbase->setUserName( username );
@@ -178,9 +179,9 @@ bool SqlTables::newTables( const QString & username, const QString & password, c
             QSqlDatabase::removeDatabase( database );
             return false;
         }
-        
 
-        QSqlQuery query( NULL, dbase );                
+
+        QSqlQuery query( NULL, dbase );
 
         // barcode_basic
         query.exec("DROP TABLE " TABLE_BASIC );
@@ -219,7 +220,7 @@ bool SqlTables::newTables( const QString & username, const QString & password, c
                    "    customer_no varchar(20) DEFAULT NULL,"
                    "    encoding_type varchar(50) DEFAULT NULL,"
                    "    article_no varchar(50) DEFAULT NULL,"
-                   "    article_no_customer varchar(50) NULL,"                   
+                   "    article_no_customer varchar(50) NULL,"
                    "    barcode_no TEXT DEFAULT NULL,"
                    "    line0 varchar(50) DEFAULT NULL,"
                    "    line1 varchar(50) DEFAULT NULL,"
@@ -255,11 +256,11 @@ bool SqlTables::newTables( const QString & username, const QString & password, c
                    ");" );
 
         dbase->close();
-        QSqlDatabase::removeDatabase( database );        
+        QSqlDatabase::removeDatabase( database );
         KMessageBox::information( 0, i18n("Created table ")+database+i18n(" successfully!") );
     } else {
         dbase->close();
-        QSqlDatabase::removeDatabase(drivers[driver]->initdb( database ));                
+        QSqlDatabase::removeDatabase(drivers[driver]->initdb( database ));
         KMessageBox::sorry( 0, i18n("Can't connect to database.") );
         return false;
     }
@@ -282,7 +283,7 @@ void SqlTables::importLabelDef()
         config->setGroup( "Definitions" );
         f = config->readEntry( "defpath", locate( "data", "kbarcode/labeldefinitions.sql" ) );
     }
-        
+
     importData( f, db );
 
     Definition::updateProducer();
@@ -304,13 +305,16 @@ void SqlTables::importData( const QString & filename, QSqlDatabase* db )
         qDebug("Can't import data, dabase not open!");
         return;
     }
-    
+
     if( filename.isEmpty() || !db->isOpen() ) // quick escape
+    {
+        KMessageBox::error( NULL, i18n("Data file for import not found. Continuing without importing data. Please check your KBarcode installation.") );
         return;
+    }
 
     QFile data( filename);
     QProgressDialog* dlg = new QProgressDialog( i18n("SQL import progress:"),  QString::null, data.size(), 0, "dlg", true );
-    
+
     if( data.open( IO_ReadOnly ) ) {
         QString s;
         QSqlQuery query( QString::null, db );
@@ -337,12 +341,12 @@ void SqlTables::loadConfig()
     KConfig* config = kapp->config();
 
     config->setGroup("SQL");
-    sqldata->username = config->readEntry("username", "root");
-    sqldata->password = config->readEntry("password", "" );
-    sqldata->hostname = config->readEntry("hostname", "localhost" );
-    sqldata->database = config->readEntry("database", "kbarcode" );
-    sqldata->driver = config->readEntry("driver", "QMYSQL3" );
-    sqldata->autoconnect = config->readBoolEntry("autoconnect", false );
+    sqldata.username = config->readEntry("username", "root");
+    sqldata.password = config->readEntry("password", "" );
+    sqldata.hostname = config->readEntry("hostname", "localhost" );
+    sqldata.database = config->readEntry("database", "kbarcode" );
+    sqldata.driver = config->readEntry("driver", "QMYSQL3" );
+    sqldata.autoconnect = config->readBoolEntry("autoconnect", false );
 }
 
 void SqlTables::saveConfig()
@@ -350,12 +354,12 @@ void SqlTables::saveConfig()
     KConfig* config = kapp->config();
 
     config->setGroup("SQL");
-    config->writeEntry("username", sqldata->username );
-    config->writeEntry("password", sqldata->password );
-    config->writeEntry("hostname", sqldata->hostname );
-    config->writeEntry("database", sqldata->database );
-    config->writeEntry("driver", sqldata->driver );
-    config->writeEntry("autoconnect", sqldata->autoconnect );
+    config->writeEntry("username", sqldata.username );
+    config->writeEntry("password", sqldata.password );
+    config->writeEntry("hostname", sqldata.hostname );
+    config->writeEntry("database", sqldata.database );
+    config->writeEntry("driver", sqldata.driver );
+    config->writeEntry("autoconnect", sqldata.autoconnect );
 
     config->sync();
 }
@@ -368,7 +372,7 @@ void SqlTables::updateTables()
      * as they have already been created with the most
      * recent database structures.
      */
-    if( sqldata->driver != "QMYSQL3" )
+    if( sqldata.driver != "QMYSQL3" )
         return;
 
     bool changed = false;
@@ -400,12 +404,12 @@ void SqlTables::updateTables()
             changed = true;
         }
 
-    // change barcode_no from varchar to TEXT (1.5.0)        
+    // change barcode_no from varchar to TEXT (1.5.0)
     QSqlQuery query3("SHOW FIELDS FROM " TABLE_CUSTOMER_TEXT );
     while( query3.next() )
         if( query3.value( 0 ).toString() == "barcode_no" && query3.value(1) == "varchar(50)" ) {
             query3.exec( "ALTER TABLE " TABLE_CUSTOMER_TEXT " CHANGE barcode_no barcode_no TEXT" );
-            changed = true;           
+            changed = true;
         }
 
     // change NUMERIC DEFAULT TO NUMERIC(10,4) (1.6.1)
@@ -420,7 +424,7 @@ void SqlTables::updateTables()
             changed = true;
         }
 
-    if( changed )      
+    if( changed )
         KMessageBox::information( 0, i18n("The SQL tables of KBarcode have changed since the last version. "
                                           "KBarcode updated them without any loss of data."  ) );
 }
@@ -428,7 +432,9 @@ void SqlTables::updateTables()
 bool SqlTables::testSettings( const QString & username, const QString & password, const QString & hostname, const QString & database, const QString & driver )
 {
     QSqlDatabase* db = QSqlDatabase::addDatabase( driver );
-
+    if( !drivers[driver] )
+      return false;
+    
     db->setDatabaseName( drivers[driver]->initdb( database ) );
 
     db->setUserName( username );
@@ -449,6 +455,36 @@ bool SqlTables::testSettings( const QString & username, const QString & password
 
 }
 
+const QString SqlTables::getBarcodeMaxLength( const QString & name )
+{
+    if( SqlTables::isConnected() )
+    {
+        QSqlQuery query("select uid, (length(barcode_no)) as LEN from " TABLE_BASIC
+            " where encoding_type = '" + name +"' ORDER by LEN DESC LIMIT 1" );
+        
+        while( query.next() ) {
+            QSqlQuery queryuid("select barcode_no from barcode_basic where uid = '"
+                + query.value( 0 ).toString() + "'" );
+            while( queryuid.next() )
+                if(!queryuid.value( 0 ).toString().isEmpty())
+                    return queryuid.value( 0 ).toString();
+        }
+        
+        QSqlQuery query1("select uid, (length(barcode_no)) as LEN from " TABLE_CUSTOMER_TEXT 
+                        " where encoding_type = '" + name +"' ORDER by LEN DESC LIMIT 1" );
+        
+        while( query1.next() ) {
+            QSqlQuery queryuid("select barcode_no from customer_text where uid = '"
+                + query1.value( 0 ).toString() + "'" );
+            while( queryuid.next() )
+                if(!queryuid.value( 0 ).toString().isEmpty())
+                    return queryuid.value( 0 ).toString();
+        }
+    }
+    
+    return "1234567";        
+}
+
 SqlWidget::SqlWidget( bool showlabel, QWidget* parent, const char* name )
     : QWidget( parent, name )
 {
@@ -521,14 +557,14 @@ SqlWidget::SqlWidget( bool showlabel, QWidget* parent, const char* name )
 
     connect( buttonTest, SIGNAL( clicked() ), this, SLOT( testSettings() ) );
 
-    mysqldata* sqldata = SqlTables::getInstance()->getData();
-    m_username->setText( sqldata->username );
-    m_password->setText( sqldata->password );
-    m_hostname->setText( sqldata->hostname );
-    m_database->setText( sqldata->database );
-    m_autoconnect->setChecked( sqldata->autoconnect );
+    const mysqldata & sqldata = SqlTables::getInstance()->getData();
+    m_username->setText( sqldata.username );
+    m_password->setText( sqldata.password );
+    m_hostname->setText( sqldata.hostname );
+    m_database->setText( sqldata.database );
+    m_autoconnect->setChecked( sqldata.autoconnect );
     for( int i = 0; i < m_driver->count(); i++ )
-        if( m_driver->text(i) == sqldata->driver )
+        if( m_driver->text(i) == sqldata.driver )
             m_driver->setCurrentItem( i );
 }
 
@@ -538,13 +574,14 @@ SqlWidget::~SqlWidget()
 
 void SqlWidget::save( bool usedb )
 {
-    mysqldata* sqldata = SqlTables::getInstance()->getData();
-    sqldata->username = m_username->text();
-    sqldata->password = m_password->text();
-    sqldata->hostname = m_hostname ->text();
-    sqldata->database = m_database->text();
-    sqldata->driver = m_driver->currentText();
-    sqldata->autoconnect = ( usedb ? m_autoconnect->isChecked() : false );
+    mysqldata sqldata = SqlTables::getInstance()->getData();
+    sqldata.username = m_username->text();
+    sqldata.password = m_password->text();
+    sqldata.hostname = m_hostname ->text();
+    sqldata.database = m_database->text();
+    sqldata.driver = m_driver->currentText();
+    sqldata.autoconnect = ( usedb ? m_autoconnect->isChecked() : false );
+    SqlTables::getInstance()->setData( sqldata );
 }
 
 void SqlWidget::testSettings()
@@ -553,8 +590,6 @@ void SqlWidget::testSettings()
     SqlTables::getInstance()->testSettings( m_username->text(), m_password->text(),
                                             m_hostname->text(), m_database->text(),
                                             m_driver->currentText() ) );
-
- 
 }
 
 const QString SqlWidget::username() const
@@ -592,3 +627,5 @@ int SqlWidget::driverCount() const
     return m_driver->count();
 }
 
+
+#include "sqltables.moc"
